// ===============================================================
// CBP_Helper2.c
// 08-09-20
// (1) Non-blocking delay function
// (2) Inverse LUT for ePuck distance sensors
// ===============================================================

#include <stdio.h>
#include <webots/robot.h>
#include <webots/compass.h>

// Function provides a non-blovking delay ------------------------
bool delay(double delay) {
  static double previous_time;
  static bool done = false;  
  static bool first_pass = true;
  
  if(first_pass){
    previous_time =  wb_robot_get_time();
    first_pass = false;
  }
      
  double current_time = wb_robot_get_time();
  //printf("     prev %f  curr = %f\n",previous_time,current_time);
    
  if(current_time - previous_time > delay) {
    previous_time = current_time;
    done = true;
    first_pass = true;
   // printf("\n");
  } else {
    done = false;  
   // printf(".");
  }
  return done;
}

typedef struct { double x; double y; } LUT;

// Function to invert the LUT of the distance sensor, to give an
// output distance
double lookUpDistance(LUT *c, double x) {
    int i;
    int n = 10;
    for( i = 0; i < n-1; i++ ) {
        if ( c[i].x <= x && c[i+1].x >= x ) {
            double diffx = x - c[i].x;
            double diffn = c[i+1].x - c[i].x;
            return c[i].y + ( c[i+1].y - c[i].y ) * diffx / diffn; 
        }
    }
    return -1; // Not in Range
}


// Compass function -----------------------------------------------
double getBearing(WbDeviceTag comp ){
  double bearing;
  const double *val = wb_compass_get_values(comp);
  double rad = atan2(val[0],val[2]);
  bearing = (180.0*rad)/M_PI - 180.0;    
  if(bearing < 0.0)
    bearing = bearing + 360.0;
  return bearing;
}

// Function to invert light sensor readings -----------------------
double invertLightSensorVals(double val) {

  double output = 4095.0 - val;
  if(output < 0.0) output = 0.0;
  return output;

}

